@@ -16,9 +16,22 @@
 #
 # Refer to the README and COPYING files for full details of the license
 #
+from __future__ import absolute_import
+"""I handle vdsm's configuration life cycle.
+
+This is achieved by utilizing modules from configurators package to:
+- Configure the machine to run vdsm after package installation.
+- Cleanup configuration before package removal.
+- Check configuration status and validity upon init.
+
+configurators interface is described below.
+"""
+
 
 from collections import deque
 import argparse
+import os
+import re
 import sys
 import traceback
 
@@ -27,23 +40,86 @@ from . import \
     expose, \
     UsageError, \
     requiresRoot
-from .configurators import \
-    certificates, \
-    CONFIGURED, \
-    InvalidConfig, \
-    InvalidRun, \
-    libvirt, \
-    NOT_CONFIGURED, \
-    sanlock, \
-    sebool
+from . import configurators
 
 
-_CONFIGURATORS = dict((m.getName(), m) for m in (
-    certificates.Certificates(),
-    libvirt.Libvirt(),
-    sanlock.Sanlock(),
-    sebool.SEBool(),
-))
+def _import_module(abspkg, mname):
+    """return imported module mname from abspkg."""
+    # TODO: use importlib once python version >= 2.7
+    pkg = "%s.%s" % (abspkg.__name__, mname)
+    return __import__(pkg, globals(), locals(), [mname], level=0)
+
+
+def _listmodules(path):
+    """Return base file names for all modules under pkg."""
+    is_module = re.compile(r"^[^_].*\.pyc?$").search
+    return set(
+        os.path.splitext(name)[0]
+        for name in os.listdir(path)
+        if is_module(name)
+    )
+
+
+_CONFIGURATORS = {}
+for module in _listmodules(configurators.__path__[0]):
+    _CONFIGURATORS[module] = _import_module(configurators, module)
+    if not hasattr(_CONFIGURATORS[module], 'name'):
+        setattr(_CONFIGURATORS[module], 'name', module)
+
+
+#
+# Configurators Interface:
+#
+# Default implementation follows;
+#
+
+def _getrequires(module):
+    """Return a set of module names required by this module.
+
+    Those modules will be included even if not provided in --module.
+    """
+    return getattr(module, 'requires', frozenset())
+
+
+def _getservices(module):
+    """Return the names of services to reload.
+
+    These services will be stopped before this configurator is called,
+    and will be started in reversed order when the configurator is done.
+    """
+    return getattr(module, 'services', ())
+
+
+def _validate(module):
+    """Return True if this module's configuration is valid.
+
+    Note: Returning False will cause vdsm to abort during initialization.
+    """
+    return getattr(module, 'validate', lambda: True)()
+
+
+def _configure(module):
+    """Prepare this module to run vdsm."""
+    getattr(module, 'configure', lambda: None)()
+
+
+def _isconfigured(module):
+    """Return state of configuration. See configurators/__init__.py
+
+    Note: returning NO will cause vdsm to abort during initialization.
+
+    Note: after configure isconfigured should return MAYBE or YES.
+    """
+    return getattr(module, 'isconfigured', lambda: configurators.NO)()
+
+
+def _removeConf(module):
+    """Cleanup vdsm's configuration."""
+    getattr(module, 'removeConf', lambda: None)()
+
+#
+# Configurators Interface End.
+#
 
 
 @expose("configure")
@@ -55,25 +131,17 @@ def configure(*args):
     Invoke with -h for complete usage.
     """
     args = _parse_args(*args)
-    configurer_to_trigger = []
 
     sys.stdout.write("\nChecking configuration status...\n\n")
-    for c in args.modules:
-        isconfigured = c.isconfigured()
-        override = args.force and isconfigured != CONFIGURED
-        if not override and not c.validate():
-            raise InvalidConfig(
-                "Configuration of %s is invalid" % c.getName()
-            )
-        if override or isconfigured == NOT_CONFIGURED:
-            configurer_to_trigger.append(c)
+    configurer_to_trigger = [c for c in args.modules
+                             if _should_configure(c, args.force)]
 
     services = []
     for c in configurer_to_trigger:
-        for s in c.getServices():
+        for s in _getservices(c):
             if service.service_status(s, False) == 0:
                 if not args.force:
-                    raise InvalidRun(
+                    raise configurators.InvalidRun(
                         "\n\nCannot configure while service '%s' is "
                         "running.\n Stop the service manually or use the "
                         "--force flag.\n" % s
@@ -85,8 +153,8 @@ def configure(*args):
 
     sys.stdout.write("\nRunning configure...\n")
     for c in configurer_to_trigger:
-        c.configure()
-        sys.stdout.write("Reconfiguration of %s is done.\n" % (c.getName(),))
+        _configure(c)
+        sys.stdout.write("Reconfiguration of %s is done.\n" % (c.name,))
 
     for s in reversed(services):
         service.service_start(s)
@@ -104,8 +172,7 @@ def isconfigured(*args):
     ret = True
     args = _parse_args(*args)
 
-    m = [c.getName() for c in args.modules
-         if c.isconfigured() == NOT_CONFIGURED]
+    m = [c.name for c in args.modules if _isconfigured(c) == configurators.NO]
 
     if m:
         sys.stdout.write(
@@ -126,7 +193,7 @@ If all modules are not configured try to use:
 (The force flag will stop the module's service and start it
 afterwards automatically to load the new configuration.)
 """
-        raise InvalidRun(msg)
+        raise configurators.InvalidRun(msg)
 
 
 @expose("validate-config")
@@ -140,7 +207,7 @@ def validate_config(*args):
     ret = True
     args = _parse_args(*args)
 
-    m = [c.getName() for c in args.modules if not c.validate()]
+    m = [c.name for c in args.modules if not _validate(c)]
 
     if m:
         sys.stdout.write(
@@ -149,7 +216,9 @@ def validate_config(*args):
         ret = False
 
     if not ret:
-        raise InvalidConfig("Config is not valid. Check conf files")
+        raise configurators.InvalidConfig(
+            "Config is not valid. Check conf files"
+        )
 
 
 @expose("remove-config")
@@ -162,21 +231,16 @@ def remove_config(*args):
     failed = False
     for c in args.modules:
         try:
-            c.removeConf()
-            sys.stdout.write(
-                "removed configuration of module %s successfully\n" %
-                c.getName()
-            )
-
+            _removeConf(c)
         except Exception:
             sys.stderr.write(
                 "can't remove configuration of module %s\n" %
-                c.getName()
+                c.name
             )
             traceback.print_exc(file=sys.stderr)
             failed = True
     if failed:
-        raise InvalidRun("Remove configuration failed")
+        raise configurators.InvalidRun("Remove configuration failed")
 
 
 def _add_dependencies(modulesNames):
@@ -186,7 +250,7 @@ def _add_dependencies(modulesNames):
     while queue:
         next_ = queue.popleft()
         try:
-            requiredNames = _CONFIGURATORS[next_].getRequires()
+            requiredNames = _getrequires(_CONFIGURATORS[next_])
         except KeyError:
             available = ', '.join(sorted(_CONFIGURATORS))
             raise UsageError(
@@ -212,8 +276,7 @@ def _sort_modules(modulesNames):
     while modulesNames:
 
         for c in modulesNames:
-            requires = _CONFIGURATORS[c].getRequires()
-            if requires.issubset(set(sortedModules)):
+            if _getrequires(_CONFIGURATORS[c]).issubset(set(sortedModules)):
                 modulesNames.remove(c)
                 sortedModules.append(c)
                 break
@@ -258,3 +321,14 @@ def _parse_args(action, *args):
     args.modules = [_CONFIGURATORS[cName] for cName in args.modules]
 
     return args
+
+
+def _should_configure(c, force):
+    configured = _isconfigured(c)
+    configure_allowed = (configured == configurators.NO or
+                         (configured == configurators.MAYBE and force))
+    if not _validate(c) and not configure_allowed:
+        raise configurators.InvalidConfig(
+            "Configuration of %s is invalid" % c.name
+        )
+    return configure_allowed
