@@ -54,7 +54,6 @@ import string
 import threading
 import time
 import vdsm.infra.zombiereaper as zombiereaper
-from M2Crypto import SSL
 
 from cpopen import CPopen
 from .config import config
@@ -62,18 +61,12 @@ from . import cmdutils
 from . import constants
 
 try:
-    # If failing to import old code, then try importing the legacy code
-    from ovirtnode import ovirtfunctions
-    persist = ovirtfunctions.ovirt_store_config
-    unpersist = ovirtfunctions.remove_config
+    from ovirt.node.utils.fs import Config
+    persist = Config().persist
+    unpersist = Config().unpersist
 except ImportError:
-    try:
-        from ovirt.node.utils.fs import Config
-        persist = Config().persist
-        unpersist = Config().unpersist
-    except ImportError:
-        persist = lambda name: None
-        unpersist = lambda name: None
+    persist = lambda name: None
+    unpersist = lambda name: None
 
 
 # Buffsize is 1K because I tested it on some use cases and 1K was fastest. If
@@ -626,7 +619,7 @@ class AsyncProc(object):
         self._poller.close()
 
 
-_ANY_CPU = ["0-%d" % (os.sysconf('SC_NPROCESSORS_ONLN') - 1)]
+_ANY_CPU = ["0-%d" % (os.sysconf('SC_NPROCESSORS_CONF') - 1)]
 _USING_CPU_AFFINITY = config.get('vars', 'cpu_affinity') != ""
 
 
@@ -1266,11 +1259,10 @@ def round(n, size):
 
 
 def create_connected_socket(host, port, sslctx=None, timeout=None):
+    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     if sslctx:
-        sock = SSL.Connection(sslctx.context)
-    else:
-        sock = socket.socket(socket.AF_INET,
-                             socket.SOCK_STREAM)
+        sock = sslctx.wrapSocket(sock)
+
     sock.settimeout(timeout)
     sock.connect((host, port))
     return sock
