@@ -22,15 +22,19 @@ import os
 import math
 import tempfile
 import logging
+import platform
 from stat import S_IROTH
 from functools import partial, wraps
 
 from nose.plugins.skip import SkipTest
 
-from testrunner import VdsmTestCase as TestCaseBase
-from testrunner import permutations, expandPermutations
-from testrunner import temporaryPath
+from testlib import VdsmTestCase as TestCaseBase
+from testlib import permutations, expandPermutations
+from testlib import temporaryPath
 
+import verify
+
+import caps
 from vdsm.utils import CommandPath, RollbackContext
 import storageTests as storage
 from storage.misc import execCmd
@@ -55,6 +59,9 @@ _tmpinitramfs = False
 
 VM_MINIMAL_UPTIME = 30
 
+_GRAPHICS_FOR_ARCH = {caps.Architecture.PPC64LE: 'vnc',
+                      caps.Architecture.X86_64: 'qxl'}
+
 
 class VDSMConnectionError(Exception):
     pass
@@ -158,7 +165,7 @@ class RunningVm(object):
 
 
 @expandPermutations
-class VirtTestBase(TestCaseBase):
+class VirtTestBase(TestCaseBase, verify.DeviceMixin):
     UPSTATES = frozenset((vmstatus.UP, vmstatus.POWERING_UP))
 
     def setUp(self):
@@ -212,6 +219,12 @@ class VirtTestBase(TestCaseBase):
         self.retryAssert(partial(self.assertVmDown, vmid),
                          timeout=10)
 
+    def _verifyDevices(self, vmId):
+        status, msg, stats = self.vdsm.getVmList(vmId)
+        self.assertEqual(status, SUCCESS, msg)
+
+        self.verifyDevicesConf(conf=stats['devices'])
+
 
 @expandPermutations
 class VirtTest(VirtTestBase):
@@ -219,7 +232,57 @@ class VirtTest(VirtTestBase):
     def testSimpleVm(self):
         customization = {'vmId': '77777777-ffff-3333-bbbb-222222222222',
                          'vmName': 'testSimpleVm',
-                         'display': 'qxl'}
+                         'display': _GRAPHICS_FOR_ARCH[platform.machine()]}
+
+        with RunningVm(self.vdsm, customization) as vm:
+            self._waitForStartup(vm, VM_MINIMAL_UPTIME)
+            self._verifyDevices(vm)
+
+    @requireKVM
+    def testComplexVm(self):
+        customization = {'vmId': '77777777-ffff-3333-bbbb-222222222222',
+                         'vmName': 'testComplexVm',
+                         'display': _GRAPHICS_FOR_ARCH[platform.machine()],
+                         'devices': [
+                             {'type': 'sound', 'device': 'ac97'},
+                             {'type': 'sound', 'device': 'ich6'},
+                             {'type': 'video', 'device': 'qxl'},
+                             {'type': 'video', 'device': 'qxl'},
+                             {'type': 'graphics', 'device': 'spice'},
+                             {'type': 'controller', 'device': 'virtio-serial'},
+                             {'type': 'controller', 'device': 'usb'},
+                             {'type': 'balloon', 'device': 'memballoon',
+                              'specParams': {'model': 'virtio'}},
+                             {'type': 'watchdog', 'device': 'wawtchdog'},
+                             {'type': 'smartcard', 'device': 'smartcard',
+                              'specParams': {'type': 'spicevmc',
+                                             'mode': 'passthrough'}},
+                             {'type': 'console', 'device': 'console'},
+                             {'nicModel': 'virtio', 'device': 'bridge',
+                              'macAddr': '52:54:00:59:F5:3F', 'network': '',
+                              'type': 'interface'},
+                             {'nicModel': 'virtio', 'device': 'bridge',
+                              'macAddr': '52:54:00:59:FF:FF', 'network': '',
+                              'type': 'interface'},
+                         ]}
+
+        status, msg, caps = self.vdsm.getVdsCapabilities()
+        self.assertEqual(status, SUCCESS, msg)
+
+        if caps['rngSources']:
+            for _ in range(0, 2):
+                customization['devices'].append(
+                    {'type': 'rng', 'model': 'virtio', 'device': 'rng',
+                     'specParams': {'source': caps['rngSources'][0]}})
+
+        with RunningVm(self.vdsm, customization) as vm:
+            self._waitForStartup(vm, VM_MINIMAL_UPTIME)
+            self._verifyDevices(vm)
+
+    @requireKVM
+    def testHeadlessVm(self):
+        customization = {'vmId': '77777777-ffff-3333-bbbb-222222222222',
+                         'vmName': 'testHeadlessVm'}
 
         with RunningVm(self.vdsm, customization) as vm:
             self._waitForStartup(vm, VM_MINIMAL_UPTIME)
@@ -234,19 +297,21 @@ class VirtTest(VirtTestBase):
         customization = {'vmId': '88888888-eeee-ffff-aaaa-111111111111',
                          'vmName': 'testVmWithStorage' + backendType,
                          'drives': drives,
-                         'display': 'vnc'}
+                         'display': _GRAPHICS_FOR_ARCH[platform.machine()]}
 
         with RollbackContext() as rollback:
             disk.createVdsmStorageLayout(conf, 3, rollback)
             with RunningVm(self.vdsm, customization) as vm:
                 self._waitForStartup(vm, VM_MINIMAL_UPTIME)
+                self._verifyDevices(vm)
 
     @requireKVM
     @permutations([['hotplugNic'], ['virtioNic'], ['smartcard'],
                    ['hotplugDisk'], ['virtioRng']])
     def testVmWithDevice(self, *devices):
         customization = {'vmId': '77777777-ffff-3333-bbbb-222222222222',
-                         'vmName': 'testVm', 'devices': [], 'display': 'vnc'}
+                         'vmName': 'testVm', 'devices': [],
+                         'display': _GRAPHICS_FOR_ARCH[platform.machine()]}
         storageLayout = storage.storageLayouts['localfs']
         diskSpecs = storage.StorageTest.generateDriveConf(storageLayout)
         pciSpecs = {'bus': '0x00', 'domain': '0x0000',
@@ -297,6 +362,7 @@ class VirtTest(VirtTestBase):
 
         with RunningVm(self.vdsm, customization) as vm:
             self._waitForStartup(vm, VM_MINIMAL_UPTIME)
+            self._verifyDevices(vm)
 
             if 'hotplugNic' in devices:
                 self.retryAssert(partial(self.vdsm.hotplugNic,
@@ -316,7 +382,7 @@ class VirtTest(VirtTestBase):
                          'devices': [],
                          'vmName':
                          'testVmWithCdrom_%s' % pathLocation,
-                         'display': 'vnc'}
+                         'display': _GRAPHICS_FOR_ARCH[platform.machine()]}
 
         # echo -n testPayload | md5sum
         # d37e46c24c78b1aed33496107afdb44b
@@ -344,6 +410,7 @@ class VirtTest(VirtTestBase):
 
             with RunningVm(self.vdsm, customization) as vm:
                 self._waitForStartup(vm, 10)
+                self._verifyDevices(vm)
 
                 status, msg, stats = self.vdsm.getVmList(vm)
                 self.assertEqual(status, SUCCESS, msg)
@@ -363,6 +430,7 @@ class VirtTest(VirtTestBase):
 
         with RunningVm(self.vdsm, customization) as vm:
             self._waitForStartup(vm, VM_MINIMAL_UPTIME)
+            self._verifyDevices(vm)
 
     @permutations([['vnc'], ['spice']])
     def testVmDefinitionGraphics(self, displayType):
@@ -374,6 +442,8 @@ class VirtTest(VirtTestBase):
 
         with RunningVm(self.vdsm, customization) as vm:
             self._waitForStartup(vm, VM_MINIMAL_UPTIME)
+            self._verifyDevices(vm)
+
             status, msg, stats = self.vdsm.getVmStats(vm)
             self.assertEqual(status, SUCCESS, msg)
             self.assertEqual(stats['displayInfo'][0]['type'],
@@ -392,6 +462,8 @@ class VirtTest(VirtTestBase):
 
         with RunningVm(self.vdsm, customization) as vm:
             self._waitForStartup(vm, VM_MINIMAL_UPTIME)
+            self._verifyDevices(vm)
+
             status, msg, stats = self.vdsm.getVmStats(vm)
             self.assertEqual(status, SUCCESS, msg)
             for dispInfo, dispType in zip(stats['displayInfo'],
@@ -403,10 +475,12 @@ class VirtTest(VirtTestBase):
     def testVmWithSla(self):
         customization = {'vmId': '99999999-aaaa-ffff-bbbb-111111111111',
                          'vmName': 'testVmWithSla',
-                         'display': 'vnc'}
+                         'display': _GRAPHICS_FOR_ARCH[platform.machine()]}
 
         with RunningVm(self.vdsm, customization) as vm:
             self._waitForStartup(vm, VM_MINIMAL_UPTIME)
+            self._verifyDevices(vm)
+
             status, msg, stats = self.vdsm.getVmStats(vm)
             self.assertEqual(status, SUCCESS, msg)
             self.vdsm.updateVmPolicy(customization['vmId'],
