@@ -60,7 +60,7 @@ def __convertDomainMetadataToTags(domain, targetVersion):
     if chkMetadata[sd.DMDK_VERSION] == int(targetVersion):
         # Switching to the newMetadata (successful upgrade), the oldMetadata
         # was cleared after all.
-        domain._metadata = chkMetadata
+        domain.replaceMetadata(chkMetadata)
         log.debug("Conversion of domain %s to tag based metadata completed, "
                   "target version = %s", domain.sdUUID, targetVersion)
     else:
@@ -180,9 +180,9 @@ def v3DomainConverter(repoPath, hostId, domain, isMsd):
                       metaOffset, vol.volUUID)
             metaContent = vol.getMetadata()
 
-            with vol._tagCreateLock:
-                newMetaSlot = domain.getVolumeMetadataSlot(
-                    vol.volUUID, blockVolume.VOLUME_MDNUMBLKS)
+            with domain.acquireVolumeMetadataSlot(
+                    vol.volUUID, blockVolume.VOLUME_MDNUMBLKS) \
+                    as newMetaSlot:
                 if newMetaSlot > metaMaxSlot:
                     raise se.NoSpaceLeftOnDomain(domain.sdUUID)
 
