@@ -45,7 +45,8 @@ else:
 
 import misc
 import fileUtils
-import zombiereaper
+import vdsm.infra.zombiereaper as zombiereaper
+import vdsm.infra.filecontrol as filecontrol
 from vdsm.compat import pickle
 from vdsm import utils
 
@@ -126,15 +127,15 @@ class CrabRPCProxy(object):
     def __init__(self, myRead, myWrite):
         self._myWrite = myWrite
         self._myRead = myRead
-        misc.setNonBlocking(self._myWrite)
-        misc.setNonBlocking(self._myRead)
+        filecontrol.set_non_blocking(self._myWrite)
+        filecontrol.set_non_blocking(self._myRead)
         self._poller = select.poll()
 
     @contextmanager
     def _poll(self, fd, events, timeout):
         self._poller.register(fd, events)
         try:
-            res = misc.NoIntrPoll(self._poller.poll, timeout)
+            res = utils.NoIntrPoll(self._poller.poll, timeout)
             for fd, event in res:
                 if event & (select.POLLERR | select.POLLHUP):
                     raise Timeout()
