@@ -18,7 +18,6 @@
 # Refer to the README and COPYING files for full details of the license
 #
 
-import thread
 import os
 import errno
 import time
@@ -36,6 +35,7 @@ import task
 from threadPool import ThreadPool
 from storage_exception import InvalidParameterException
 from vdsm import constants
+from vdsm import utils
 
 __author__ = "ayalb"
 __date__ = "$Mar 9, 2009 5:25:07 PM$"
@@ -269,6 +269,7 @@ class HSM_MailMonitor(threading.Thread):
         self._sendMail()  # Clear outgoing mailbox
         threading.Thread.__init__(self)
         self.daemon = True
+        self.name = "mailbox.HSMMonitor"
         self.start()
 
     def _initMailbox(self):
@@ -425,6 +426,8 @@ class HSM_MailMonitor(threading.Thread):
                         MESSAGES_PER_MAILBOX,
                         repr(self._outgoingMail[start:end])))
 
+    @utils.traceback(on=log.name,
+                     msg="Unhandled exception in HSM_MailMonitor thread")
     def run(self):
         try:
             failures = 0
@@ -567,8 +570,8 @@ class SPM_MailMonitor:
                         'conv=notrunc',
                         'count=1'
                         ]
-        self._outLock = thread.allocate_lock()
-        self._inLock = thread.allocate_lock()
+        self._outLock = threading.Lock()
+        self._inLock = threading.Lock()
         # Clear outgoing mail
         self.log.debug("SPM_MailMonitor - clearing outgoing mail, command is: "
                        "%s", self._outCmd)
@@ -578,7 +581,10 @@ class SPM_MailMonitor:
             self.log.warning("SPM_MailMonitor couldn't clear outgoing mail, "
                              "dd failed")
 
-        thread.start_new_thread(self.run, (self, ))
+        t = threading.Thread(target=self.run)
+        t.daemon = True
+        t.name = "mailbox.SPMMonitor"
+        t.start()
         self.log.debug('SPM_MailMonitor created for pool %s' % self._poolID)
 
     def stop(self):
@@ -591,21 +597,19 @@ class SPM_MailMonitor:
         return self._numHosts
 
     def setMaxHostID(self, newMaxId):
-        self._inLock.acquire()
-        self._outLock.acquire()
-        diff = newMaxId - self._numHosts
-        if diff > 0:
-            delta = MAILBOX_SIZE * diff * "\0"
-            self._outgoingMail += delta
-            self._incomingMail += delta
-        elif diff < 0:
-            delta = MAILBOX_SIZE * diff
-            self._outgoingMail = self._outgoingMail[:-delta]
-            self._incomingMail = self._incomingMail[:-delta]
-        self._numHosts = newMaxId
-        self._outMailLen = MAILBOX_SIZE * self._numHosts
-        self._outLock.release()
-        self._inLock.release()
+        with self._inLock:
+            with self._outLock:
+                diff = newMaxId - self._numHosts
+                if diff > 0:
+                    delta = MAILBOX_SIZE * diff * "\0"
+                    self._outgoingMail += delta
+                    self._incomingMail += delta
+                elif diff < 0:
+                    delta = MAILBOX_SIZE * diff
+                    self._outgoingMail = self._outgoingMail[:-delta]
+                    self._incomingMail = self._incomingMail[:-delta]
+                self._numHosts = newMaxId
+                self._outMailLen = MAILBOX_SIZE * self._numHosts
 
     def _validateMailbox(self, mailbox, mailboxIndex):
         chkStart = MAILBOX_SIZE - CHECKSUM_BYTES
@@ -778,7 +782,9 @@ class SPM_MailMonitor:
         finally:
             self._outLock.release()
 
-    def run(self, *args):
+    @utils.traceback(on=log.name,
+                     msg="Unhandled exception in SPM_MailMonitor thread")
+    def run(self):
         try:
             while not self._stop:
                 try:
